module fx2lp_slaveFIFO2b_loopback_fpga_top(
//	reset_n_in,
	fdata,  
        faddr,  
        slrd,   
        slwr,   
        sloe,              
        flagd,  
        flaga, 
        done,	
        clk,    
        clk_out
);

//input reset_n_in;
inout [15:0]fdata;
input flaga;
input flagd;
input clk;

output clk_out; 
output [1:0]faddr;
output sloe;
output slwr;
output slrd;
output done;

reg slrd_n;
reg slwr_n;
reg sloe_n;

reg [15:0] fifo_data_in;
wire [15:0] fifo_data_out;
reg [15:0] data_out;

reg done_d;
reg [3:0]wait_s;

wire clk_out_0;
wire clk_out_90;
wire clk_out_180;
wire clk_out_270;
reg [1:0]faddr_n;

parameter [1:0] loop_back_idle       = 2'd0;
parameter [1:0] loop_back_read       = 2'd1;
parameter [1:0] loop_back_wait_flagd = 2'd2;
parameter [1:0] loop_back_write      = 2'd3;

reg [1:0]current_loop_back_state;
reg [1:0]next_loop_back_state;

wire fifo_full;
wire fifo_empty;
wire fifo_push;
wire fifo_pop;


ODDR2 oddr2_inst
	(
	 .D0(1'b1),
	 .D1(1'b0),
	 .CE(1'b1),
	 .C0(clk_out_180),  
	 .C1(!clk_out_180), 
	 .R (1'b0),
	 .S (1'b0),
	 .Q (clk_out)
	);
	
		
clk_wiz_v3_6 clk_inst  	                                   //PLL
	(
	 .CLK_IN1  (clk),
	 .CLK_OUT1 (clk_out_0),		
	 .CLK_OUT2 (clk_out_90),
	 .CLK_OUT3 (clk_out_180),
	 .CLK_OUT4 (clk_out_270),
	 .RESET    (1'b0/*reset_n_in*/),
	 .LOCKED   (reset_n)
		);

//assign clk_out_0 = clk;

assign slwr = slwr_n;
assign slrd = slrd_n;
assign sloe = sloe_n;
assign faddr = faddr_n;

assign fdata = data_out;
assign done = done_d;


always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)begin
		done_d <= 1'b0;
 	end else if(wait_s == 4'd10) begin
		done_d <= 1'b1;

	end
end	

always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)begin
		wait_s <= 4'd0;
 	end else if(wait_s < 4'd10) begin
		wait_s <= wait_s + 1'b1;

	end
end	



always@(*)begin
	if((current_loop_back_state == loop_back_idle) | (current_loop_back_state == loop_back_read))begin
		faddr_n = 2'b00;
	end else begin
		faddr_n = 2'b10;
	end
end	



//read control signal generation
always@(*)begin
	if((current_loop_back_state == loop_back_read) & (flaga == 1'b1))begin
		slrd_n = 1'b0;
		sloe_n = 1'b0;
	end else begin
		slrd_n = 1'b1;
		sloe_n = 1'b1;
	end
end	

assign fifo_push = ((slrd_n == 1'b0) & (fifo_full == 1'b0));

always@(*)begin
	if(slrd_n == 1'b0)
		fifo_data_in = fdata;
	else
		fifo_data_in = "00000000";
	
end	

//write control signal generation
always@(*)begin
	if((current_loop_back_state == loop_back_write) & (flagd == 1'b1) & (fifo_empty == 1'b0))
		slwr_n <= 1'b0;
	else
		slwr_n <= 1'b1;
end

assign fifo_pop = ((slwr_n == 1'b0) & (fifo_empty == 1'b0));

//loopback mode state machine 
always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)
      		current_loop_back_state <= loop_back_idle;
        else
                current_loop_back_state <= next_loop_back_state;
end

//LoopBack mode state machine combo
always@(*) begin
	next_loop_back_state = current_loop_back_state;
	case(current_loop_back_state)
		loop_back_idle:begin
			if(flaga == 1'b1)
				next_loop_back_state = loop_back_read;
			else
				next_loop_back_state = loop_back_idle;
		end
		loop_back_read:begin
			if(flaga == 1'b0)
				next_loop_back_state = loop_back_wait_flagd;
			else
				next_loop_back_state = loop_back_read;
		end
		loop_back_wait_flagd:begin
			if(flagd == 1'b1)
				next_loop_back_state = loop_back_write;
			else 
				next_loop_back_state = loop_back_wait_flagd;
		end
		loop_back_write:begin
			if((flagd == 1'b0) | (fifo_empty == 1'b1))
				next_loop_back_state = loop_back_idle;
			else 
		 		next_loop_back_state = loop_back_write;
		end
		default: 
			next_loop_back_state = loop_back_idle;
	endcase		
end

always@(*)begin
	if(slwr_n == 1'b1)
		data_out = 16'dz;
	else
		data_out = fifo_data_out;
end

//fifo instantiation for loopback
fifo_512x8 fifo_inst
	(
	 .din	     (fifo_data_in),	
	 .write_busy (fifo_push),
         .fifo_full  (fifo_full), 
         .dout	     (fifo_data_out),
         .read_busy  (fifo_pop),
         .fifo_empty (fifo_empty),
         .fifo_clk   (clk_out_0),
         .reset_   (reset_n),
         .fifo_flush (current_loop_back_state == loop_back_idle)
	);


endmodule
