module fx2lp_slaveFIFO2b_streamOUT_fpga_top(
	fdata,                                //  FIFO data lines.
        faddr,                                //  FIFO select line
        slrd,                                 //  Read control lin
        slwr,                                 //  Write control li
        sloe,                                 //  Slave Output Enable control
        flagd,                                //  EP6 full flag
        flaga,                                //  EP2 empty flag
	done,
        clk,                                   //  Interface Clock
	clk_out,
	pkt_end
);

inout [15:0]fdata;
input flaga;
input flagd;
input clk;

output [1:0]faddr;
output sloe;
output slwr;
output slrd;
output pkt_end;
output clk_out;
output done;

reg slrd_n;
reg slwr_n;
reg sloe_n;

wire [15:0] data_out;

wire clk_out_0;
wire clk_out_90;
wire clk_out_180;
wire clk_out_270;
reg [1:0]faddr_n;

reg done_d;
reg [3:0]wait_s;

parameter [1:0] stream_out_idle       = 2'd0;
parameter [1:0] stream_out_read       = 2'd1;

reg [1:0]current_stream_out_state;
reg [1:0]next_stream_out_state;

ODDR2 oddr2_inst
	(
	 .D0(1'b1),
	 .D1(1'b0),
	 .CE(1'b1),
	 .C0(clk_out_180),  
	 .C1(!clk_out_180), 
	 .R (1'b0),
	 .S (1'b0),
	 .Q (clk_out)
	);
	
		
clk_wiz_v3_6 clk_inst  	                                   //PLL
	(
	 .CLK_IN1  (clk),
	 .CLK_OUT1 (clk_out_0),		
	 .CLK_OUT2 (clk_out_90),
	 .CLK_OUT3 (clk_out_180),
	 .CLK_OUT4 (clk_out_270),
	 .RESET    (1'b0),
	 .LOCKED   (reset_n)
		);


assign slwr = slwr_n;
assign slrd = slrd_n;
assign sloe = sloe_n;
assign faddr = 2'b00;
assign pkt_end = 1'b1;
assign data_out = fdata;

assign done = done_d;


always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)begin
		done_d <= 1'b0;
 	end else if(wait_s == 4'd10) begin
		done_d <= 1'b1;

	end
end	

always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)begin
		wait_s <= 4'd0;
 	end else if(wait_s < 4'd10) begin
		wait_s <= wait_s + 1'b1;

	end
end	


//read control signal generation
always@(*)begin
	if((current_stream_out_state == stream_out_read) & (flaga == 1'b1))begin
		slrd_n = 1'b0;
		sloe_n = 1'b0;
	end else begin
		slrd_n = 1'b1;
		sloe_n = 1'b1;
	end
end	



//streamOUt mode state machine 
always@(posedge clk_out_0, negedge reset_n) begin
	if(reset_n == 1'b0)
      		current_stream_out_state <= stream_out_idle;
        else
                current_stream_out_state <= next_stream_out_state;
end

//streamOUT mode state machine combo
always@(*) begin
	next_stream_out_state = current_stream_out_state;
	case(current_stream_out_state)
		stream_out_idle:begin
			if(flaga == 1'b1)
				next_stream_out_state = stream_out_read;
			else
				next_stream_out_state = stream_out_idle;
		end
		stream_out_read:begin
			if(flaga == 1'b0)
				next_stream_out_state = stream_out_idle;
			else
				next_stream_out_state = stream_out_read;
		end
		default: 
			next_stream_out_state = stream_out_idle;
	endcase		
end

endmodule
